@@ -25,11 +25,13 @@ void dbg_(const char *fcn, long line, const char *fmt, ...);
 #endif	
 
 int init(const char *file, const char *cardname);
-int state_lock(const char *file, int lock, int timeout);
+int state_lock(const char *file, int timeout);
+int state_unlock(int fd, const char *file);
 int save_state(const char *file, const char *cardname);
 int load_state(const char *file, const char *initfile, const char *cardname,
 	       int do_init);
 int power(const char *argv[], int argc);
+int monitor(const char *name);
 int state_daemon(const char *file, const char *cardname, int period,
 		 const char *pidfile);
 int state_daemon_kill(const char *pidfile, const char *cmd);
