@@ -115,6 +115,9 @@ fail:
 
 int main(int argc, char *argv[])
 {
+	if (!isatty(fileno(stdin)))
+		return 0;
+
 	setlocale(LC_ALL, "");
 #ifdef ENABLE_NLS_IN_CURSES
 	textdomain(PACKAGE);
@@ -130,6 +133,6 @@ int main(int argc, char *argv[])
 
 	mainloop();
 
-	shutdown();
+	app_shutdown();
 	return 0;
 }
