@@ -26,6 +26,7 @@
 #include <alsa/asoundlib.h>
 #include <getopt.h>
 #include <signal.h>
+#include <assert.h>
 #include "aconfig.h"
 #include "gettext.h"
 
@@ -82,7 +83,7 @@ static int info = 0;
  * main routine
  */
 
-static struct option long_option[] = {
+static const struct option long_option[] = {
 	{"port", 1, NULL, 'p'},
 	{"source", 1, NULL, 's'},
 	{"dest", 1, NULL, 'd'},
@@ -166,7 +167,7 @@ static void usage(void)
 	printf(_("  server mode: aseqnet [-options]\n"));
 	printf(_("  client mode: aseqnet [-options] server_host\n"));
 	printf(_("options:\n"));
-	printf(_("  -p,--port # : sepcify TCP port (digit or service name)\n"));
+	printf(_("  -p,--port # : specify TCP port (digit or service name)\n"));
 	printf(_("  -s,--source addr : read from given addr (client:port)\n"));
 	printf(_("  -d,--dest addr : write to given addr (client:port)\n"));
 	printf(_("  -v, --verbose : print verbose messages\n"));
